/*
 * Project: S.Ha.R.K.
 *
 * Coordinators:
 *   Giorgio Buttazzo    <giorgio@sssup.it>
 *   Paolo Gai           <pj@gandalf.sssup.it>
 *
 * Authors     :
 *   Trimarchi Michael   <trimarchi@gandalf.sssup.it>
 *   (see the web pages for full authors list)
 *
 * ReTiS Lab (Scuola Superiore S.Anna - Pisa - Italy)
 *
 * http://www.sssup.it
 * http://retis.sssup.it
 * http://shark.sssup.it
 */

/*
 * Copyright (C) 2000 Giorgio Buttazzo, Paolo Gai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 * CVS :        $Id: server-task.c,v 1.4 2004/05/26 13:44:55 trimarchi Exp $
 */

#include "ll/i386/64bit.h"
#include <kernel/kern.h>
#include <modules/sem.h>
#include <modules/hartport.h>
#include <modules/cabs.h>
#include <string.h>
#include "fsf_contract.h"
#include "fsf_server.h"
#include "fsf_service_task.h"
#include "message.h"

TASK    service_task() {
  
  struct mess message;
  PORT rec,tra;

  rec = port_connect("CHANW",sizeof(struct mess),STREAM,READ);
  
  tra = port_connect("CHANR",sizeof(struct mess),STREAM,WRITE);
  
  while(1) {
    // wait for renegotiation
    port_receive(rec,&message,BLOCK);
    switch (message.type) {
      
    case NEGOTIATE_CONTRACT:
      if (negotiate_contract(&message.contract, &message.server)==FSF_ERR_CONTRACT_REJECTED)
	message.server=-1;
			 
    break;
    case RENEGOTIATE_CONTRACT:
      if (renegotiate_contract(&message.contract, message.server)==FSF_ERR_CONTRACT_REJECTED)
	message.server=-1;
      break;			 
			 
    default :
      break;

    }
    // send response server is -1 if the operation fail
    port_send(tra,&message,BLOCK);

    task_endcycle();
    
  }
}


static __inline void set_contract_parameter(void) {

  int i=0;
  TIME T,Q;

   for (i=0; i<current_server; i++) {
      mul32div32to32(MAX_BANDWIDTH,server_list[i].Cmin,server_list[i].U,T);
      if (T > server_list[i].Tmin ) {
         server_list[i].actual_budget = server_list[i].Cmin;
         server_list[i].actual_period = T;
      #ifdef FSF_DEBUG
         kern_printf("(1 - Q %ld T %ld)", server_list[i].actual_budget, server_list[i].actual_period);
      #endif
         if (server_list[i].d_equals_t == TRUE)
           adjust_SERVER_budget(server_list[i].server,server_list[i].Cmin, T, T);
	 else
	   adjust_SERVER_budget(server_list[i].server,server_list[i].Cmin, T, server_list[i].deadline);

      } else {
        mul32div32to32(server_list[i].Tmin,server_list[i].U,MAX_BANDWIDTH,Q);
        server_list[i].actual_budget = Q;
        server_list[i].actual_period = server_list[i].Tmin;
      #ifdef FSF_DEBUG
         kern_printf("(2 - Q %ld T %ld)", server_list[i].actual_budget, server_list[i].actual_period);
      #endif
                    
         if (server_list[i].d_equals_t == TRUE)
	   adjust_SERVER_budget(server_list[i].server,server_list[i].Cmin, server_list[i].Tmin, server_list[i].Tmin);
         else
           adjust_SERVER_budget(server_list[i].server,server_list[i].Cmin, server_list[i].Tmin, server_list[i].deadline);                                                                                                        
     }
                                                                                                                             
     server_list[i].U=server_list[i].Umin;
   }
 
}


int renegotiate_contract 
  (const fsf_contract_parameters_t *new_contract,
   fsf_server_id_t                 server)
{
  
  #ifdef FSF_DEBUG
    kern_printf("(Renegotiate for server %d)",server);
  #endif

  if (!new_contract)
    return FSF_ERR_NOT_INITIALIZED;

  if (server < 0)
    return FSF_ERR_INVALID_SERVER;
 
   // change the parameter
   relink_contract_to_server(new_contract, server); 
   if (recalculate_contract(fsf_max_bw)==-1)  {
       return FSF_ERR_CREATE_SERVER;
   }
   
   set_contract_parameter();

   return 0;
}

int negotiate_contract
  (const fsf_contract_parameters_t *contract,
   fsf_server_id_t                 *server)
{
  /* Check if contract is initialized */
  if (!contract) return FSF_ERR_NOT_INITIALIZED;

  /* Admission Test */
  if (FSF_ADMISSION_TEST_IS_ENABLED)
    if (add_contract(contract))
      return FSF_ERR_CONTRACT_REJECTED; 

  /* SERVER => BUDGET */    
  set_SERVER_budget_from_contract(contract,server);

#ifdef FSF_DEBUG
  kern_printf("(New Server %d)",*server);
#endif

  if (*server >= 0) {
    link_contract_to_server(contract,*server);
    if (recalculate_contract(fsf_max_bw)==-1)  {
      return FSF_ERR_CREATE_SERVER;
    }
    set_contract_parameter();
#ifdef  FSF_DEBUG
    kern_printf("(Adjust budget)");
#endif    
    
  }
  else  {
    return FSF_ERR_CREATE_SERVER;
  }
  
  return 0;

}


#if defined MICHAEL
int recalculate_contract(bandwidth_t U) {
  bandwidth_t current_bandwidth;
  unsigned int temp_U;
  int        Qt;
  int isok=0;
  int i=0;
  
  #ifdef FSF_DEBUG
    int temp;
  
    kern_printf("(Recalculate contract)");
  #endif

  /* The current bandwidth is the min bandwidth */
  //current_bandwidth=SERVER_return_bandwidth(fsf_server_level);
  #ifdef FSF_DEBUG
     kern_printf("(nserver %d)", current_server);
  #endif  
  
  do  {
    current_bandwidth=0;
    Qt=0;
    for (i=0; i<current_server; i++) {
      if (server_list[i].Qs!=0 && server_list[i].U<server_list[i].Umax)
         Qt+=server_list[i].Qs; 
       current_bandwidth+=server_list[i].U;
    }

    #ifdef FSF_DEBUG
    kern_printf("(Total Quality %d)", Qt);
    #endif
    isok=1;
    for (i=0; i<current_server; i++) {
      if (server_list[i].Qs!=0 && server_list[i].U<server_list[i].Umax) {
	temp_U=server_list[i].U;
        server_list[i].U=U-current_bandwidth;
	mul32div32to32(server_list[i].U, server_list[i].Qs, Qt, server_list[i].U);
	temp_U+=server_list[i].U;
	
        if (temp_U<=server_list[i].Umin) {
	   server_list[i].U=server_list[i].Umin;
	} else if (temp_U>server_list[i].Umax)  {
	   server_list[i].U=server_list[i].Umax;
	   isok=0;
	} else server_list[i].U=temp_U; 
 	
#ifdef FSF_DEBUG
	mul32div32to32(server_list[i].U,100, MAX_BANDWIDTH, temp);
        kern_printf("(Server %d bw %d)", server_list[i].server, temp);
#endif 
      } 
    }  

  } while (!isok);
  
 return 0;
}
#endif
